C                                   THIS PROGRAM WAS CODED BY M.SAITO
C *********************************************************************
C
      SUBROUTINE BUTFIL( X, Y, NN, DT, F1, F2, F3, F4 )
C
C *********************************************************************
C
C     DRIVER OF BUTTERWORTH DIGITAL FILTER
C       X(NN)  : INPUT TIME SERIES
C       Y(NN)  : OUTPUT TIME SERIES (MAY BE EQUIVALENT X )
C       NN     : 
C       DT     : TIME INCREMENT (SEC)
C       F1     : LOW-CUT FREQUENCY  (HZ)(IF BANDPASS, F1 IS NOT USED)
C       F2     : LOW-CUT FREQUENCY  (HZ)
C       F3     : HIGH-CUT FREQUENCY  (HZ)
C       F4     : HIGH-CUT FREQUENCY  (HZ)
C
C                       *--------------*
C                      /F2            F3+
C                     /                  +
C             ------ *                    *-------> FREQENCY(HZ)
C                   (F1)                  F4
C
      DIMENSION X(*),Y(*)
      DIMENSION H(50)
C
      DATA   AP,AS / 0.5, 5.0 / 
C
C
C --- CALCULATE H(M) --------------------------------------------------
C
      IF     ( F1.EQ.0.0 .OR. F2.EQ.0.0 ) THEN
        CALL BUTLOP( H, M, GN, N,           F3*DT, F4*DT, AP, AS )
      ELSE IF( F3.EQ.0.0 .OR. F4.EQ.0.0 ) THEN
        CALL BUTHIP( H, M, GN, N,           F2*DT, F1*DT, AP, AS )
      ELSE
        CALL BUTPAS( H, M, GN, N,    F2*DT, F3*DT, F4*DT, AP, AS )
      END IF
CHECK WRITE(*,*) 'N=',N,'   M=',M
C
C --- FILTERING -------------------------------------------------------
C
C     *** FORWARD ***
C
      CALL TANDEM(X,Y,NN,H,M,1)
      DO 300 I=1,NN
        Y(I)=Y(I)*GN
  300 CONTINUE
C
C     *** REVIERSE ***
C
      CALL TANDEM(Y,Y,NN,H,M,-1)
      DO 400 I=1,NN
        Y(I)=Y(I)*GN
  400 CONTINUE
C
      RETURN
      END
C *********************************************************************
C
      SUBROUTINE CHBFIL( X, Y, NN, DT, F1, F2, F3, F4 )
C
C *********************************************************************
C
C     DRIVER OF CHEBYSHEV DIGITAL FILTER
C       X(NN)  : INPUT TIME SERIES
C       Y(NN)  : OUTPUT TIME SERIES (MAY BE EQUIVALENT X )
C       NN     : 
C       DT     : TIME INCREMENT (SEC)
C       F1     : LOW-CUT FREQUENCY  (HZ)(IF BANDPASS, F1 IS NOT USED)
C       F2     : LOW-CUT FREQUENCY  (HZ)
C       F3     : HIGH-CUT FREQUENCY  (HZ)
C       F4     : HIGH-CUT FREQUENCY  (HZ)
C
C                       *--------------*
C                      /F2            F3+
C                     /                  +
C             ------ *                    *-------> FREQENCY(HZ)
C                   (F1)                  F4
C
      DIMENSION X(*),Y(*)
      DIMENSION H(50)
C
      DATA   AP,AS / 0.5, 5.0 / 
C
C
C --- CALCULATE H(M) --------------------------------------------------
C
      IF     ( F1.EQ.0.0 .OR. F2.EQ.0.0 ) THEN
        CALL CHBLOP( H, M, GN, N, EPS,          F3*DT, F4*DT, AP, AS )
      ELSE IF( F3.EQ.0.0 .OR. F4.EQ.0.0 ) THEN
        CALL CHBHIP( H, M, GN, N, EPS,          F2*DT, F1*DT, AP, AS )
      ELSE
        CALL CHBPAS( H, M, GN, N, EPS,   F2*DT, F3*DT, F4*DT, AP, AS )
      END IF
CHECK WRITE(*,*) 'N=',N,'   M=',M
C
C --- FILTERING -------------------------------------------------------
C
C     *** FORWARD ***
C
      CALL TANDEM(X,Y,NN,H,M,1)
      DO 300 I=1,NN
        Y(I)=Y(I)*GN
  300 CONTINUE
C
C     *** REVIERSE ***
C
      CALL TANDEM(Y,Y,NN,H,M,-1)
      DO 400 I=1,NN
        Y(I)=Y(I)*GN
  400 CONTINUE
C
      RETURN
      END
C *********************************************************************
C
      SUBROUTINE SAITO( X, Y, NN, DT, ITYPE, IDIR, F1, F2, F3, F4 )
C
C *********************************************************************
C
C     DRIVER OF BUTTERWORTH DIGITAL FILTER
C       X(NN)  : INPUT TIME SERIES
C                / IDIR=2 -> AMPLITUDE
C       Y(NN)  : OUTPUT TIME SERIES (MAY BE EQUIVALENT X )
C                / IDIR=2 -> PHASE
C       NN     : 
C       DT     : TIME INCREMENT (SEC)
C       ITYPE  : X1=LOW PASS     X2=HIGHT PASS     X3=BAND PASS
C                0X=BUTTERWORTH  1X=CHEBYSHEV      2X=CHEBYSHEV(2)
C       IDIR   : 1=NORMAL        -1=REVIERSE        0=BOTH
C
C       F1     : LOW-PASS   *           *           *          (HZ)
C       F2     : LOW-PASS   BAND-PASS   *           BAND-STOP  (HZ)
C       F3     : *          BAND-PASS   HIGH-PASS   BAND-STOP  (HZ)
C       F4     : *          BAND-PASS   HIGH-PASS   BAND-STOP  (HZ)
C
C                       *--------------*
C                      /F2            F3+
C                     /                  +
C             ------ *                    *-------> FREQENCY(HZ)
C                    F1                   F4
C
      DIMENSION X(*),Y(*)
      DIMENSION H(50)
C
C       AP     : MAX. ATTENUATION IN PASS BAND
C       AS     : MIN. ATTENUATION IN STOP BAND
C
      DATA   AP,AS / 0.5, 5.0 / 
C
      DO 100 I=1,NN
        Y(I)=X(I)
 100  CONTINUE
C
C --- CALCULATE H(M) --------------------------------------------------
C
      write(6,*) 'SAITO FILTER'
      IF     (ITYPE.EQ.00) THEN
        RETURN
      ELSE IF(ITYPE.EQ.01) THEN
        write(6,*) 'BUTLOP'
        CALL BUTLOP( H, M, GN, N,           F3*DT, F4*DT, AP, AS )
      ELSE IF(ITYPE.EQ.02) THEN
        write(6,*) 'BUTHIP'
        CALL BUTHIP( H, M, GN, N,           F2*DT, F1*DT, AP, AS )
      ELSE IF(ITYPE.EQ.03) THEN
        write(6,*) 'BUTPAS'
        CALL BUTPAS( H, M, GN, N,    F2*DT, F3*DT, F4*DT, AP, AS )
      ELSE IF(ITYPE.EQ.13) THEN
        write(6,*) 'CHBPAS'
        CALL CHBPAS( H, M, GN, N, E, F2*DT, F3*DT, F4*DT, AP, AS )
      ELSE IF(ITYPE.EQ.23) THEN
        write(6,*) 'TCHPAS'
        CALL TCHPAS( H, M, GN, N, E, F2*DT, F3*DT, F4*DT, AP, AS )
      ELSE
        WRITE(*,*) 'SAITO : ERROR( NOT SURPORTED !! )'
        RETURN
      END IF
CHECK WRITE(*,*) 'N=',N,'   M=',M
C
C --- FILTER RESPONSE -------------------------------------------------
C 
      IF( IDIR.EQ.2 ) THEN
        DF = 1.0 / DT / REAL(NN)
        CALL RECRES( H,M,GN,0.0,DF*DT,X,Y,NN/2+1 )
        RETURN
      END IF
C
C --- FILTERING -------------------------------------------------------
C
C     *** FORWARD ***
C
      IF( IDIR.GE.0 ) THEN
        CALL TANDEM(X,Y,NN,H,M,1)
        DO 300 I=1,NN
          Y(I)=Y(I)*GN
  300   CONTINUE
      END IF
C
C     *** REVIERSE ***
C
      IF( IDIR.LE.0 ) THEN
        CALL TANDEM(Y,Y,NN,H,M,-1)
        DO 400 I=1,NN
          Y(I)=Y(I)*GN
  400   CONTINUE
      END IF
C
      RETURN
      END
C *********************************************************************
C
      SUBROUTINE RECFIL(X,Y,N,H,NML)
C
C *********************************************************************
C                                 CODED BY M.SAITO 1975.12.06
C
C      RECURSIVE FILTERING : F(Z)=(1+A*Z+AA*Z**2)/(1+B*Z+BB*Z**2)
C
C      ARGUMENTS
C        X      : INPUT TIME SERIES
C        Y      : OUTPUT TIME SERIES (MAY BE EQUIVALENT TO X)
C        N      : LENGTH OF X & Y
C        H      : FILTER COEFFICIENTS ; H(1)=A, H(2)=AA, H(3)=B, H(4)=BB
C        NML    : >0 ;FOR NORMAL DIRECTION FILTERING
C                 <0 ;FOR REVERSE DIRECTION FILTERING
C
      DIMENSION X(N),Y(N),H(4)
      IF(   N.LE.0 )  GO TO 4
      IF( NML.GE.0 )  GO TO 1
C **** REVERSE FILTERING
      J  = N
      JD =-1
        GO TO 2
C **** NORMAL FILTERING
    1 J  = 1
      JD = 1
C
    2 A  = H(1)
      AA = H(2)
      B  = H(3)
      BB = H(4)
      U1 = 0.
      U2 = 0.
      V1 = 0.
      V2 = 0.
C **** FILTERING
      DO 3 I=1,N
        U3 = U2
        U2 = U1
        U1 = X(J)
        V3 = V2
        V2 = V1
        V1 = U1+A*U2+AA*U3-B*V2-BB*V3
        Y(J) = V1
        J  = J+JD
    3 CONTINUE
C **** EXIT
      RETURN
C **** ERROR
    4 WRITE(6,5) N
    5 FORMAT(//1X,5('?'),3X,'(RECFIL)',3X,'INVALID INPUT',3X,'N =',I5,
     *       3X,5('?')//)
      RETURN
      END
C *********************************************************************
C
      SUBROUTINE TANDEM(X,Y,N,H,M,NML)
C
C *********************************************************************
C                              CODED BY M.SAITO 1975.12.06
C
C      RECURSIVE FILTERING IN SERIES
C
C      ARGUMENTS
C        X      : INPUT TIME SERIES
C        Y      : OUTPUT TIME SERIES  (MAY BE EQUIVALENT TO X)
C        N      : LENGTH OF X & Y
C        H      : COEFFICIENTS OF FILTER
C        M      : ORDER OF FILTER
C        NML    : >0 ; FOR NORMAL  DIRECTION FILTERING
C                 <0 ;     REVERSE DIRECTION FILTERING
C
C      SUBROUTINE REQUIRED ; RECFIL
C
      DIMENSION X(N),Y(N),H(4)
      IF( N.LE.0 .OR. M.LE.0 )  GO TO 2
C **** 1-ST CALL
      CALL  RECFIL(X,Y,N,H,NML)
      IF( M.LE.1 ) RETURN
C **** 2-ND AND AFTER
      DO 1 I=2,M
        CALL  RECFIL(Y,Y,N,H(I*4-3),NML)
    1 CONTINUE
C
      RETURN
C **** ERROR
    2 WRITE(6,3)N,M
    3 FORMAT(//1X,5('?'),3X,'(TANDEM)',3X,'INVALID',3X,'N =',I5,
     *       3X,'M =',I5,5('?')//)
      RETURN
      END
C *********************************************************************
C
      SUBROUTINE RECRES(H,M,GN,FM,DF,G,P,N)
C
C *********************************************************************
C                                CODED BY M.SAITO 1975.12.13
C
C     RECURSIVE FILTER FREQUENCY RESPONSE FUNCTION
C
C     ARGUMENTS
C       H      : COEFFICIENTS OF FILTER
C       M      : ORDER OF FILTER
C       GN     : GAIN FACTER
C       FM     : MIN FREQUENCY (NON-DIMENSIONAL)
C       DF     : STEPY ; FREQ=FM,FM+DF,.,.,.,FM+(N-1)DF    : POWER TRANSFER FUNCTION
C    PHASE D   :ELAY
C       N      : NUMBER OF G & P
C
      DIMENSION     H(4),G(N),P(N)
      DATA          PI/3.141593/ ,PI2/6.283185/
C
      IF(M.LE.0 .OR. N.LE.0 ) GO TO 4
      K  = M*4
      C  = GN**2.
      CS = COS(FM*PI2)
      SN = SIN(FM*PI2)
      DC =-2.*SIN(DF*PI)**2.
      DS = SIN(DF*PI2)
C
      DO 3 I=1,N
        GG = C
        PP = 0.
C
        DO 1 J=1,K,4
C***** NUMERATOR
          A  = (H(J+1)+1.)*CS+H(J)
          B  = (H(J+1)-1.)*SN
          GG = GG*(A**2.+B**2.)
          IF( GG.LE.0. ) GO TO 2
          PP = PP+ATAN2(B,A)
C***** DENOMINATOR
          A  = (H(J+3)+1.)*CS+H(J+2)
          B  = (H(J+3)-1.)*SN
          GG = GG/(A**2.+B**2.)
          PP = PP-ATAN2(B,A)
C***** REDUCE PHASE
          IF( ABS(PP).GT.PI ) PP = PP-SIGN(PI2,PP)
    1   CONTINUE
C
    2   G(I) = GG
        P(I) = PP
C***** STEP OF FREQUENCY
        W  = CS
        CS = CS*DC-SN*DS+CS
        SN = W *DS+SN*DC+SN
        W  = (1.-CS**2.-SN**2.)*0.5
        CS = CS*W+CS
        SN = SN*W+SN 
    3 CONTINUE
C
      RETURN
C***** ERROR
    4 WRITE(6,5) M,N
    5 FORMAT(//1X,'?????',3X,'(RECRES)',3X,'INVALID INPUT',3X,
     *         'M=',I5,3X,'N=',I5,3X,'?????'//)
      RETURN
      END
C *********************************************************************
C
      SUBROUTINE BUTLOP(H,M,GN,N,FP,FS,AP,AS) 
C
C *********************************************************************
C                                   CODED BY M.SAITO 1975.12.16
C
C      BUTTERWORTH  LOW PASS FILTER COEFFICENT
C
C      ARGUMENTS
C        H      : FILTER COEFFICIENT
C        M      : ORDER OF FILTER (M=(N+1)/2)
C        GN     : GAIN FACTER
C        N      : DEGREE OF BUTTERWORTH FUNCTION
C        FP     : PASS BAND FREQUENCY (NON-DIMENSIONAL)
C        FS     : STOP BAND FREQUENCY
C        AP     : MAX. ATTENUATION IN PASS BAND
C        AS     : MIN. ATTENUATION IN STOP BAND
C
      DIMENSION H(4)
      DATA PI/3.141593/,HP/1.570796/
C
      WP = AMIN1(ABS(FP),ABS(FS))*PI
      WS = AMAX1(ABS(FP),ABS(FS))*PI
      IF( WP.EQ.0. .OR. WS.EQ.WP .OR. WP.GE.HP ) GO TO 100
C****  DFIFRMINE N & C
      TP = TAN(WP)
      TS = TAN(WS)
      PA = AMIN1(ABS(AP),ABS(AS)) 
      SA = AMAX1(ABS(AP),ABS(AS)) 
      IF( PA.EQ.0. ) PA = 0.5 
      IF( SA.EQ.0. ) SA = 5.
      N  = MAX0(2,INT(ABS(ALOG(SA/PA)/ALOG(TP/TS))+0.5))
      CC = EXP(ALOG(PA*SA)/FLOAT(N))/(TP*TS)
      C  = SQRT(CC)
C
      DP = HP/FLOAT(N)
      M  = N/2 
      K  = M*4 
      G  = 1.
      FJ = 1. 
      C2 = 2.*(1.-C)*(1.+C)
C
      DO 1 J=1,K,4
        SJ = COS(DP*FJ)**2.
        TJ = SIN(DP*FJ) 
        FJ = FJ+2.
        A  = 1./((C+TJ)**2.+SJ) 
        G  = G*A 
          H( J ) = 2. 
          H(J+1) = 1. 
          H(J+2) = C2*A 
          H(J+3) = ((C-TJ)**2.+SJ)*A 
    1 CONTINUE
C****  EXIT
      GN = G
      IF( MOD(N,2).EQ.0 ) RETURN
C****  FOR ODD N
      M  = M+1 
        GN = G/(C+1.) 
          H(K+1) = 1. 
          H(K+2) = 0. 
          H(K+3) = (1.-C)/(1.+C)
          H(K+4) = 0. 
      RETURN
C****  ERROR
  100 WRITE(6,101) FP,FS
  101 FORMAT(/1X,5('?'),'   (BUTLOP)   INVALID INPUT  FP=',
     *       1PF14.6,3X,'FS =',F14.6,3X,5('?')//) 
      RETURN
      END 
C *********************************************************************
C
      SUBROUTINE BUTHIP(H,M,GN,N,FP,FS,AP,AS) 
C
C *********************************************************************
C                                   CODED BY M.SAITO 1976.01.07
C
C      BUTTERWORTH HIGH PASS FILTER COEFFICENT
C
C      ARGUMENTS
C        H      : FILTER COEFFICIENT
C        M      : ORDER OF FILTER (M=(N+1)/2)
C        GN     : GAIN FACTER
C        N      : DEGREE OF BUTTERWORTH FUNCTION
C        FP     : PASS BAND FREQUENCY (NON-DIMENSIONAL)
C        FS     : STOP BAND FREQUENCY
C        AP     : MAX. ATTENUATION IN PASS BAND
C        AS     : MIN. ATTENUATION IN STOP BAND
C
      DIMENSION H(4)
      DATA PI/3.141593/,HP/1.570796/
C
      WP = AMIN1(ABS(FP),ABS(FS))*PI
      WS = AMAX1(ABS(FP),ABS(FS))*PI
      IF( WS.EQ.0. .OR. WS.EQ.WP .OR. WP.GE.HP ) GO TO 100
C****  DFIFRMINE N & C
      TP = TAN(WP)
      TS = TAN(WS)
      PA = AMIN1(ABS(AP),ABS(AS)) 
      SA = AMAX1(ABS(AP),ABS(AS)) 
      IF( PA.EQ.0. ) PA = 0.5 
      IF( SA.EQ.0. ) SA = 5.
      N  = MAX0(2,INT(ABS(ALOG(SA/PA)/ALOG(TP/TS))+0.5))
      CC = EXP(ALOG(PA*SA)/FLOAT(N))*TP*TS    
      C  = SQRT(CC)
C
      DP = HP/FLOAT(N)
      M  = N/2 
      K  = M*4 
      G  = 1.
      FJ = 1. 
      C2 =-2.*(1.-C)*(1.+C) 
      DO 1 J=1,K,4
        SJ = COS(DP*FJ)**2.
        TJ = SIN(DP*FJ) 
        FJ = FJ+2.
        A  = 1./((C+TJ)**2.+SJ) 
        G  = G*A 
          H( J ) =-2.
          H(J+1) = 1. 
          H(J+2) = C2*A 
          H(J+3) = ((C-TJ)**2.+SJ)*A 
    1 CONTINUE
C****  EXIT
      GN = G
      IF( MOD(N,2).EQ.0 ) RETURN
C****  FOR ODD N
      M  = M+1 
        GN = G/(C+1.) 
          H(K+1) =-1.
          H(K+2) = 0. 
          H(K+3) = (C-1.)/(1.+C)
          H(K+4) = 0. 
      RETURN
C****  ERROR
  100 WRITE(6,101) FP,FS
  101 FORMAT(/1X,5('?'),'   (BUTLOP)   INVALID INPUT  FP=',
     *       1PF14.6,3X,'FS =',F14.6,3X,5('?')//) 
      RETURN
      END 
C *********************************************************************
C
      SUBROUTINE BUTPAS(H,M,GN,N,FL,FH,FS,AP,AS) 
C
C *********************************************************************
C                                   CODED BY M.SAITO 1976.01.07
C
C      BUTTERWORTH BAND PASS FILTER COEFFICENT
C
C      ARGUMENTS
C        H      : FILTER COEFFICIENT
C        M      : ORDER OF FILTER 
C        GN     : GAIN FACTER
C        N      : DEGREE OF BUTTERWORTH FUNCTION
C        FL     : LOW FREQUENCY CUT-OFF (NON-DIMENSIONAL)
C        FH     : HIGH FREQUENCY CUT-OFF
C        FS     : STOP BAND FREQUENCY
C        AP     : MAX. ATTENUATION IN PASS BAND
C        AS     : MIN. ATTENUATION IN STOP BAND
C
      COMPLEX R,DJ,CQ 
      DIMENSION H(4),R(2) 
      DATA PI/3.141593/,HP/1.570796/
C
      WL = AMIN1(ABS(FL),ABS(FH))*PI
      WH = AMAX1(ABS(FL),ABS(FH))*PI
      WS = ABS(FS)*PI 
      IF( WL.EQ.0. .OR. WL.EQ.WH .OR. WH.GE.HP .OR. WS.EQ.0. .OR. 
     *    WS.GE.HP .OR. (WS-WL)*(WS-WH).LE.0. ) GO TO 100 
      CLH= 1./(COS(WL)*COS(WH))
      DP = SIN(WH-WL)*CLH 
      WW = TAN(WL)*TAN(WH)
      TS = TAN(WS)
      DS = ABS(TS-WW/TS)
      PA = AMIN1(ABS(AP),ABS(AS)) 
      SA = AMAX1(ABS(AP),ABS(AS)) 
      IF( PA.EQ.0. ) PA = 0.5 
      IF( SA.EQ.0. ) SA = 5.
      N  = MAX0(2,INT(ABS(ALOG(PA/SA)/ALOG(DP/DS))+0.5))
      CC = EXP(ALOG(PA*SA)/FLOAT(N))/(DP*DS)
      C  = SQRT(CC)
C
      WW = WW*CC
      DP = HP/FLOAT(N)
      K  = N/2 
      M  = K*2 
      L  = 0 
      G  = 1.
      FJ = 1.
C
      DO 2 J=1,K
        DJ = CMPLX(COS(DP*FJ),SIN(DP*FJ))*(0.5,0.0) 
        FJ = FJ+2.
        CQ = CSQRT(DJ**2.+CMPLX(WW,0.0))
          R(1) = DJ+CQ
          R(2) = DJ-CQ
          G  = G*CC
C
          DO 1 I=1,2    
            RE = REAL(R(I))**2.
            RI = AIMAG(R(I))
            A  = 1./((C+RI)**2.+RE) 
            G  = G*A 
              H(L+1) = 0. 
              H(L+2) =-1.
              H(L+3) = 2.*((RI-C)*(RI+C)+RE)*A
              H(L+4) = ((RI-C)**2.+RE)*A 
              L = L+4 
    1   CONTINUE
C
    2 CONTINUE
C****  EXIT
      GN = G
      IF( N.EQ.M )   RETURN
C****  FOR ODD N
      M  = M+1 
      WPC= CC*COS(WH-WL)*CLH 
      WMC=-CC*COS(WH+WL)*CLH
         A  = 1./(WPC+C)
         GN = G*C*A
           H(L+1) = 0. 
           H(L+2) =-1.
           H(L+3) = 2.*WMC*A       
           H(L+4) = (WPC-C)*A
      RETURN
C**** ERROR
  100 WRITE(6,101) FL,FH,FS 
  101 FORMAT(/1X,5('?'),'   (BUTBPS)   INVALID INPUT   FL =',
     *       1PF14.6,3X,'FH =',F14.6,3X,'FS =',F14.6,3X,5('?')//) 
      RETURN
      END 
C *********************************************************************
C
      SUBROUTINE CHBPAS(H,M,GN,N,EPS,FL,FH,FS,AP,AS)
C
C *********************************************************************
C                                   CODED BY M.SAITO 1977.11.01
C
C      CHEBYSHEV BAND-PASS FILTER COEFFICIENT
C
C      ARGUMENTS
C        H      : FILTER COEFFICIENT
C        M      : ORDER OF FILTER
C        GN     : GAIN FACTER
C        N      : DEGREE OF CHEBYSHEV POLYNOMIAL
C        EPS    : RIPPLE
C        FL     : LOW FREQUENCY CUT-OFF (NON-DIMENSIONAL)
C        FH     : HIGH FREQUENCY CUT-OFF
C        FS     : STOP BAND FREQUENCY
C        AP     : MAX. ATTENUATION IN PASS BAND
C        AS     : MIN. ATTENUATION IN STOP BAND
C
      COMPLEX R(2),DJ,CQ
      DIMENSION H(4)
      DATA PI/3.141593/,HP/1.570796/
C
      WL = AMIN1(ABS(FL),ABS(FH))*PI
      WH = AMAX1(ABS(FL),ABS(FH))*PI
      WS = ABS(FS)*PI
      IF( WL.EQ.WH .OR. WL.EQ.0. .OR. WH.GE.HP .OR. WS.EQ.0. .OR.
     *    WS.GE.HP .OR. (WS-WL)*(WS-WH).LE.0. ) GO TO 100
C **** DETERMINE N,EPS & C
      CLH= COS(WL)*COS(WH)
      C  = CLH/SIN(WH-WL)
      CC = C*C
      WW = TAN(WL)*TAN(WH)*CC
      WS = TAN(WS)*C
      DS = ABS(WS-WW/WS)
      TS = ALOG(DS+SQRT((DS-1.)*(DS+1.)))
      PA = AMIN1(ABS(AP),ABS(AS))
      SA = AMAX1(ABS(AP),ABS(AS))
      IF( PA.EQ.0. ) PA = 0.1
      IF( SA.EQ.0. ) SA = 5.
      PS = PA/SA
      N  = MAX0(2,IFIX(ABS(ALOG((1.+SQRT((1.-PS)*(1.+PS)))/PS)/TS)+0.5))
      FN = N
      EPS= SQRT(PA*SA/COSH(TS*FN))
C
      K  = N/2
      M  = K*2
      L  = 0
      FJ = 1.
      G  = 1./(EPS*FLOAT(2**(N-1)))
      PS = ALOG((1.+SQRT(1.+EPS**2.))/EPS)/FN
      CH = COSH(PS)
      SH = SINH(PS)
      DP = HP/FN
C
      DO 2 J=1,K
        DJ = CMPLX(COS(DP*FJ)*CH,SIN(DP*FJ)*SH)*0.5
        FJ = FJ+2.
        CQ = CSQRT(DJ**2.+WW)
        R(1) = DJ+CQ
        R(2) = DJ-CQ
        G  = G*CC
C
        DO 1 I=1,2
          RE = REAL(R(I))**2.
          RI = AIMAG(R(I))
          A  = 1./((C+RI)**2.+RE)
          G  = G*A
            H(L+1) = 0.
            H(L+2) =-1.
            H(L+3) = 2.*((RI-C)*(RI+C)+RE)*A
            H(L+4) = ((RI-C)**2.+RE)*A
            L=L+4
    1   CONTINUE
C
    2 CONTINUE
C **** EXIT
      GN = G
      IF( N.EQ.M ) RETURN
C **** FOR ODD N
      M  = M+1
      WPC= CC*COS(WH-WL)/CLH
      WMC=-CC*COS(WH+WL)/CLH
          A  = 1./(WPC+C*SH)
          GN = G*C*A
            H(L+1) = 0.
            H(L+2) =-1.
            H(L+3) = 2.*WMC*A
            H(L+4) = (WPC-C*SH)*A
      RETURN
C **** ERROR
  100 WRITE(6,101) FL,FH,FS
  101 FORMAT(/1X,5('?'),'   (CHBPAS)   INVALID INPUT   FL =',
     *       E14.6,3X,'FH =',E14.6,3X,'FS =',E14.6,3X,5('?')//)
      RETURN
      END
C *********************************************************************
C
      SUBROUTINE CHBLOP(H,M,GN,N,EPS,FP,FS,AP,AS)
C
C *********************************************************************
C                                   CODED BY M.SAITO 1977.11.01
C
C     CHEBYSHEV LOW-PASS FILTER COEFFICIENT
C
C     ARGUMENTS
C      H   : FILTER COEFFICIENT
C      M   : ORDER OF FILTER (M=(N+1/2))
C      GN  : GAIN FACTOR
C      N   : DEGREE OF CHEBYSHEV POLYNOMIAL
C      EPS : RIPPLE
C      FP  : PASS BAND FREQUENCY (NON-DIMENSIONAL)
C      FS  : STOP BAND FREQUENCY
C      AP  : MAX. ATTENUATION IN PASS BAND
C      AS  : MIN. ATTENUATION IN STOP BAND
C
      DIMENSION H(4)
      DATA PI/3.141593/,HP/1.570796/
C
      WP=AMIN1(ABS(FP),ABS(FS))*PI
      WS=AMAX1(ABS(FP),ABS(FS))*PI
      IF(WP.EQ.WS .OR. WP.EQ.0. .OR. WS.GE.HP ) GO TO 100
C      DETERMINE N,EPS & C
      C=  1.0 / TAN(WP)
      OS= C*TAN(WS)
      TS= ALOG(OS+SQRT((OS+1.)*SIN(WS-WP)/(COS(WS)*SIN(WP))))
      PA= AMIN1(ABS(AP),ABS(AS))
      SA= AMAX1(ABS(AP),ABS(AS))
      IF(PA.EQ.0.) PA=0.1
      IF(SA.EQ.0.) SA=5.
      PS=PA/SA
      N= MAX0(2,IFIX(ABS(ALOG((1.+SQRT((1.-PS)*(1.+PS)))/PS)/TS)+0.5))
      FN=N
      EPS= SQRT(PA*SA/COSH(TS*FN))
C
      M=N/2
      K=M*4
      G=1./(EPS*FLOAT(2**(N-1)))
      PS=ALOG((1.+SQRT(1.+EPS**2.))/EPS)/FN
      CH= COSH(PS)
      SH= SINH(PS)
      DP=HP/FN
      FJ=1.
C
      DO 1 J=1,K,4
       SJ= (COS(DP*FJ)*CH)**2.
       TJ= SIN(DP*FJ)*SH
       FJ= FJ+2.
       A = 1./((C+TJ)**2.+SJ)
       G = G*A
       H( J )= 2.
       H(J+1)= 1.
       H(J+2)= 2.*((TJ-C)*(TJ+C)+SJ)*A
       H(J+3)= ((TJ-C)**2.+SJ)*A
    1 CONTINUE
C****  EXIT
      GN=G
      IF(MOD(N,2).EQ.0) RETURN
C**** FOR ODD N
      M=M+1
      GN=G/(SH+C)
      H(K+1)=1.
      H(K+2)=0.
      H(K+3)=(SH-C)/(SH+C)
      H(K+4)=0.
      RETURN
C**** ERROR
  100 WRITE(6,101) FP,FS
  101 FORMAT(/1X,5('?'),' (CHBLOP) INVALID INPUT  FP=',
     *       1PF14.6,3X,'FS=',E14.6,3X,5('?')//)
      RETURN
      END
C *********************************************************************
C
      SUBROUTINE CHBHIP(H,M,GN,N,EPS,FP,FS,AP,AS)
C
C *********************************************************************
C                                   CODED BY M.SAITO 1977.11.01
C
C     CHEBYSHEV HIGH-PASS FILTER COEFFICIENT
C
C     ARGUMENTS
C      H   : FILTER COEFFICIENT
C      M   : ORDER OF FILTER (M=(N+1/2))
C      GN  : GAIN FACTOR
C      N   : DEGREE OF CHEBYSHEV POLYNOMIAL
C      EPS : RIPPLE
C      FP  : PASS BAND FREQUENCY (NON-DIMENSIONAL)
C      FS  : STOP BAND FREQUENCY
C      AP  : MAX. ATTENUATION IN PASS BAND
C      AS  : MIN. ATTENUATION IN STOP BAND
C
      DIMENSION H(4)
      DATA PI/3.141593/,HP/1.570796/
C
      WP=AMAX1(ABS(FP),ABS(FS))*PI
      WS=AMIN1(ABS(FP),ABS(FS))*PI
      IF(WP.EQ.WS .OR. WS.EQ.0. .OR. WP.GE.HP ) GO TO 100
C      DETERMINE N,EPS & C
      C=  TAN(WP)
      OS= C/TAN(WS)
      TS= ALOG(OS+SQRT((OS+1.)*SIN(WP-WS)/(COS(WP)*SIN(WS))))
      PA= AMIN1(ABS(AP),ABS(AS))
      SA= AMAX1(ABS(AP),ABS(AS))
      IF(PA.EQ.0.) PA=0.1
      IF(SA.EQ.0.) SA=5.
      PS=PA/SA
      N= MAX0(2,IFIX(ABS(ALOG((1.+SQRT((1.-PS)*(1.+PS)))/PS)/TS)+0.5))
      FN=N
      EPS= SQRT(PA*SA/COSH(TS*FN))
C
      M=N/2
      K=M*4
      G=1./(EPS*FLOAT(2**(N-1)))
      PS=ALOG((1.+SQRT(1.+EPS**2.))/EPS)/FN
      CH= COSH(PS)
      SH= SINH(PS)
      DP=HP/FN
      FJ=1.
C
      DO 1 J=1,K,4
       SJ= (COS(DP*FJ)*CH)**2.
       TJ= SIN(DP*FJ)*SH
       FJ= FJ+2.
       A = 1./((C+TJ)**2.+SJ)
       G = G*A
       H( J )=-2.
       H(J+1)= 1.
       H(J+2)= 2.*((C-TJ)*(C+TJ)-SJ)*A
       H(J+3)= ((C-TJ)**2.+SJ)*A
    1 CONTINUE
C****  EXIT
      GN=G
      IF(MOD(N,2).EQ.0) RETURN
C**** FOR ODD N
      M=M+1
      GN=G/(C+SH)
      H(K+1)=-1.
      H(K+2)= 0.
      H(K+3)=(C-SH)/(C+SH)
      H(K+4)=0.
      RETURN
C**** ERROR
  100 WRITE(6,101) FP,FS
  101 FORMAT(/1X,5('?'),' (CHBHIP) INVALID INPUT  FP=',
     *       1PF14.6,3X,'FS=',E14.6,3X,5('?')//)
      RETURN
      END
C *********************************************************************
C
      SUBROUTINE TCHPAS(H,M,GN,N,EPS,FL,FH,FS,AP,AS)
C
C *********************************************************************
C                                   CODED BY M.SAITO 1975.12.16
C
C      CHEBYSHEV BAND-PASS FILTER COEFFICIENT  (EQUI-RIPPLE STOP BAND)
C
C      ARGUMENTS
C        H      : FILTER COEFFICIENT
C        M      : ORDER OF FILTER
C        GN     : GAIN FACTER
C        N      : ORDER OF CHEBYSHEV POLYNOMIAL
C        EPS    : RIPPLE
C        FL     : LOW FREQUENCY CUT-OFF (NON-DIMENSIONAL)
C        FH     : HIGH FREQUENCY CUT-OFF
C        FS     : STOP BAND FREQUENCY
C        AP     : MAX. ATTENUATION IN PASS BAND
C        AS     : MIN. ATTENUATION IN STOP BAND
C
      DIMENSION  H(4),R(2)
      COMPLEX  Q(2),OJ,CQ
      DATA  PI/3.141593/,HP/1.570796/
C
      WL = AMIN1(ABS(FL),ABS(FH))*PI
      WH = AMAX1(ABS(FL),ABS(FH))*PI
      WS = ABS(FS*PI)
      IF( WL.EQ.WH .OR. WL.EQ.0. .OR. WH.GE.HP .OR. WS.EQ.0. .OR.
     &    WS.GE.HP .OR. (WS-WL)*(WS-WH).LE.0. ) GO TO 100
C **** DETERMINE N,EPS & C
      TL = TAN(WL)
      TH = TAN(WH)
      TS = TAN(WS)
      C  = 1./ABS(TS-TL*TH/TS)
      CC = C**2.
      WW = CC*TH*TL
      CLH= COS(WL)*COS(WH)
      OP = CLH/(C*SIN(WH-WL))
      PA = AMIN1(ABS(AP),ABS(AS))
      SA = AMAX1(ABS(AP),ABS(AS))
      IF( PA.EQ.0. )  PA = 0.5
      IF( SA.EQ.0. )  SA = 10.
      TP = ALOG(OP+SQRT((OP-1.)*(OP+1.)))
      PS = PA/SA
      N  = MAX0(2,IFIX(ABS(ALOG((1.+SQRT((1.-PS)*(1.+PS)))/PS)/TP)+0.5))
      FN = N
      EPS= SQRT(1./(PA*SA*COSH(TP*FN)))
C
      K  = N/2
      M  = K*2
      L  = 0
      FJ = 1.
      G  = 1.
      PS = ALOG((1.+SQRT(1.+EPS**2.))/EPS)/FN
      CH = COSH(PS)
      SH = SINH(PS)
      DP = HP/FN
C
      DO  2  J=1,K
        RJ = COS(DP*FJ)
        SJ = RJ*CH
        TJ = SIN(DP*FJ)*SH
        ST = SJ**2.+TJ**2.
        RJ = RJ**2.
        R(1) = ((1.+SQRT(1.+4.*WW*RJ))*0.5)**2./RJ
        R(2) = WW**2./R(1)
        OJ = CMPLX(SJ/ST,TJ/ST)*0.5
        FJ = FJ+2.
        CQ = CSQRT(OJ**2.+WW)
        Q(1) = OJ+CQ
        Q(2) = OJ-CQ
        G  = G*RJ/ST
C
        DO  1  I=1,2
          SJ =  REAL(Q(I))**2.
          TJ = AIMAG(Q(I))
          A  = 1./((C+TJ)**2.+SJ)
          G  = G*(CC+R(I))*A
            H(L+1) =-2.*(CC-R(I))/(CC+R(I))
            H(L+2) = 1.
            H(L+3) = 2.*((TJ+C)*(TJ-C)+SJ)*A
            H(L+4) = ((C-TJ)**2.+SJ)*A
          L  = L+4
    1   CONTINUE
C
    2 CONTINUE
C
      GN = G
      IF( MOD(N,2).EQ.0 )  RETURN
C **** ODD N
      M  = M+1
      WPC= CC*COS(WH-WL)/CLH
      WMC=-CC*COS(WH+WL)/CLH
         A  = 1./(WPC+C/SH)
         GN = GN*C*A/SH
           H(L+1) = 0.
           H(L+2) = -1.
           H(L+3) = 2.*WMC*A
           H(L+4) = (WPC-C/SH)*A
      RETURN
C **** ERROR
  100 WRITE(6,101) FL,FH,FS
  101 FORMAT(/1X,5('?'),3X,'(TCHPAS)   INVALID INPUT   FL =',1PE14.6,
     *       3X,'FH =',E14.6,3X,'FS =',E14.6,3X,5('?')/)
      RETURN
      END
